//--------------------------------------//
// Bleeder                              //
// Players bleed where they take damage //
//--------------------------------------//
class B expands Mutator;

var bool Initialized;
var Pawn PlayerList[255];
var BleederEffectSpawner PlayerBleederList[255], besBufferList[255];

// Re-setup pawn id's if a new player enters, courtesy of [KiX] ProfessorGib
function ModifyPlayer(Pawn other)
{       
	ListPlayers();
	Level.bDropDetail = False;
	if (NextMutator != None) NextMutator.ModifyPlayer(other);
}

// Assigns each pawn a unique id, with help from examples by [KiX] ProfessorGib 
function ListPlayers()
{
	local Pawn P;
	local int i, j;

	// save players to list
	i = 0;
	for (P = Level.PawnList; P != None; P = P.NextPawn)
	{
		PlayerList[i] = P;
		i++;
	}
	// reassign index entries of players' bleeder and numbleeder lists
	// first wipe buffer lists
	for (i = 0; i < 255; i++) besBufferList[i] = None;
	// now scan current lists and resort into proper order based on pawn list
	for(i = 0; i < 255; i++)
	{
		// make sure we don't overstep the valid bounds within the playerlist
		if (PlayerList[i] == None) break;
		// give a valid pawn, find the bleeders which go to it (if any) and save them to the appropriate buffer list
		for (j = 0; j < 255; j++)
		{
			// check if player isn't bleeding on us
			if (PlayerBleederList[j] == None) break;
			// we have a match
			if (PlayerBleederList[j].Owner == PlayerList[i])
			{
				besBufferList[i] = PlayerBleederList[j];
			}
		}
	}
	// now to wipe the actual lists and copy buffers to them
	for (i = 0; i < j; i++)
		{
			PlayerBleederList[i] = besBufferList[i];
		}
}

function PostBeginPlay()
{
	local int i;

	if (Initialized) return;
	else Initialized = True;

	// init all the BleederEffectSpawner's
	for (i = 0; i < 255; i++)
	{
		PlayerBleederList[i] = None;
	}
	
	Level.Game.RegisterDamageMutator(Self);

	Super.PostBeginPlay();
}

function ScoreKill(Pawn Killer, Pawn Other)
{
	local int i, j;
	local BleederEffectSpawner BES, tBES;

	// find the dead man in our list
	for (i = 0; i < 255; i++)
	{
		if ((PlayerList[i] == Other) || (PlayerList[i] == None)) break;
	}
	// paranoia check that we found the player (no reason we shouldn't)
	if ((i != 255) && (PlayerList[i] != None))
	{
		// check if there are any BleederEffectSpawns attached to this player
		if (PlayerBleederList[i] != None)
		{
			// delete and destroy all bleeder spawners for this player
			tBES = PlayerBleederList[i];
			PlayerBleederList[i] = None;
			besBufferList[i] = None;
			while (tBES != None)
			{
				BES = tBES.NextSpawner;
				tBES.NumSpawns = 0; tBES.SpawnsLeft = 0;
				tBES.StopSpawner();
				tBES.Destroy();
				tBES = BES;
			}
		}
	}

	Super.ScoreKill(Killer, Other);
}

function MutatorTakeDamage(out int ActualDamage, Pawn Victim, Pawn InstigatedBy, out Vector HitLocation, out Vector Momentum, name DamageType)
{
	local BleederEffectSpawner BES;
	local class<Effects> EffS;
	local int i, j;

	if (ActualDamage > 0)
	{
		for (i = 0; i < 255; i++)
		{
			if (PlayerList[i] == None) break;
			if (PlayerList[i] == Victim)
			{
				BES = Spawn(class'BleederEffectSpawner', Victim, , HitLocation, Victim.Rotation);
				if (BES != None)
				{
					BES.InitSpawner();
					if (ActualDamage > Victim.health)
						j = Victim.health + 5;
					else
						j = ActualDamage + 10;
					if (Victim.isA('Mercenary') || Victim.isA('Slith') || Victim.isA('Fly') ||
						Victim.isA('Pupae') || Victim.isA('TeradexMaiden') || Victim.isA('Spinner') ||
						Victim.isA('MMMercenary') || Victim.isA('MMSlith') || Victim.isA('MMFly') ||
						Victim.isA('MMPupae'))
					{
						if (ActualDamage < 10)
						{
							EffS = class'UT_BloodDrop';
							j = j + 25;
						}
						else
						if (ActualDamage < 15)
						{
							EffS = class'BloodSpurt';
							j = j + 15;
						}
						else
						if (ActualDamage < 25)
							EffS = class'GreenBloodPuff';
						else
						if (ActualDamage < 60)
							EffS = class'UT_GreenBloodPuff';
						else
							EffS = class'GreenBloodSpray';
					}
					else
					{
						if (ActualDamage < 10)
						{
							EffS = class'UT_BloodDrop';
							j = j + 25;
						}
						else
						if (ActualDamage < 15)
						{
							EffS = class'BloodSpurt';
							j = j + 15;
						}
						else
						if (ActualDamage < 26)
						{
							EffS = class'UT_BloodTrail';
							j = 2;
						}
						else
						if (ActualDamage < 40)
							EffS = class'UT_BigBloodPuff';
						else
						if (ActualDamage < 60)
							EffS = class'UT_BloodPuff';
						else
						if (ActualDamage < 75)
							EffS = class'UT_BloodHit';
						else
							EffS = class'BloodSpray';
					}
// Log("ActualDamage = "$ActualDamage$"  Effect = "$EffS$"  Victim Health = "$Victim.Health$"  default Health = "$Victim.default.Health$"  Victim Name = "$Victim.GetHumanName());
					BES.SetSpawnProperties(0.5, j, j, EffS, true);
					if (PlayerBleederList[i] == None) PlayerBleederList[i] = BES;
					else PlayerBleederList[i] = BES.AddToList(PlayerBleederList[i]);
					BES = None;
				}
				break;
			}
		}
	}

	Super.MutatorTakeDamage(ActualDamage, Victim, InstigatedBy, HitLocation, Momentum, DamageType);
}

defaultproperties
{
}
